/**********************************************************************
*  File:    HOST.H                                                    *
*  Purpose: Comtrol Hostess i Programmers' Toolkit Definitions file   *
*  Date:    February 15, 1990                                         *
*  Author:  Howard D. Straus                                          *
*  Release: 1.00 - Original release                                   *
***********************************************************************
*                         COPYRIGHT NOTICE                            *
*=====================================================================*
*   Copyright 1990, 1991 by Howard D. Straus and Comtrol Corporation. *
*   All rights reserved.  Use or reproduction by any means of any     *
*   portion of this code contrary to the developers license agreement *
*   is expressly prohibited.                                          *
***********************************************************************/

#define N_BUFFERS       200     /* number of message buffers */

#define MAX_BOARD_NO    7       /* number of predefined board addresses */
#define COMM_BUFF_SIZE  500     /* maximum size of communications buffer */
#define MAX_DATA        BUF_SIZE /* maximum size of data buffer */
#define SET_TIME_SIZE   6       /* SET_TIME message data size */
#define ENTER_DEBUGGER_SIZE 1   /* ENTER_DEBUGGER message data size */

#define HELP_FILE               1
#define LINE_TABLE      2
#define DEVICE_TABLE    3

#define BIT0        (1 <<  0)
#define BIT1        (1 <<  1)
#define BIT2        (1 <<  2)
#define BIT3        (1 <<  3)
#define BIT4        (1 <<  4)
#define BIT5        (1 <<  5)
#define BIT6        (1 <<  6)
#define BIT7        (1 <<  7)
#define BIT8        (1 <<  8)
#define BIT9        (1 <<  9)
#define BIT10       (1 << 10)
#define BIT11       (1 << 11)
#define BIT12       (1 << 12)
#define BIT13       (1 << 13)
#define BIT14       (1 << 14)
#define BIT15       (1 << 15)

#ifndef NULL
#define NULL            0
#endif
#define TRUE            1
#define FALSE           0
#define ECHO            TRUE
#define NO_ECHO         FALSE
#define FUNCT           0x100   /* designates function key */
#define ESC             27      /* ESCape character */

#define THROWAWAY_BYTES 640     /* .EXE file bytes before code */
#define MAILBOX_OFFSET  0xFFF0  /* location of mailbox in dual-ported RAM */
#define HIGH_PAGE       0x10000L
#define SCC_MAP_OFFSET  (0x1EL + HIGH_PAGE)

#define DEFAULT_BASE_ADDRESS 0xF00000L
#define DEFAULT_IRQ_NO  15
#define DEFAULT_PORT    0x218
#define DEFAULT_PORT_SPEED 9600

#define ENABLE_DP_RAM               0
#define DISABLE_DP_RAM              BIT2

#define HCB_CTL_REGS_PORT_OFFSET        0
#define HCB_CTL_REGS_SELECT_PORT_OFFSET 1
#define HCB_MEM_ENABLE_PORT_OFFSET      1
#define HCB_START_PORT_OFFSET           2
#define HCB_RESET_PORT_OFFSET           3

#define CTL_REG_1_SELECT            1     /* select DP RAM addr above 1Mb */
#define PC_MEM_XFERS                0     /* select 8-bit memory xfers */
#define AT_MEM_XFERS                BIT7  /* select 16-bit memory xfers */

#define CTL_REG_2_SELECT            2     /* for addressing < 1 Mbyte */

#define CTL_REG_3_SELECT            8   /* select DP window size and offset */
#define DP_MEM_OFFSET_SELECT        4     /* select 64K DP RAM offset */
#define HOST_DP_WINDOW_SZ_SELECT (4 << 3) /* select 64K DP RAM page */

#define CTL_REG_4_SELECT            16
#define IRQ3_SELECT                 0     /* select IRQ3 */
#define IRQ4_SELECT                 1     /* select IRQ4 */
#define IRQ5_SELECT                 2     /* select IRQ5 */
#define IRQ9_SELECT                 3     /* select IRQ9 */
#define IRQ10_SELECT                4     /* select IRQ10 */
#define IRQ11_SELECT                5     /* select IRQ11 */
#define IRQ12_SELECT                6     /* select IRQ12 */
#define IRQ15_SELECT                7     /* select IRQ13 */

#define SCC_MAP                     0x1E  /* port availability bit map */

#define HCB_RESET_WORD_OFFSET   0x10000L
#define HCB_ACK_VALUE           0x55AA

#define INTCTL_LO       0x20            /* PIC address for IRQ 0 - 7 */
#define INTMASK_LO              (INTCTL_LO + 1) /* PIC mask register IRQ 0 - 7 */
#define INTCTL_HI       0xA0            /* PIC address for IRQ 8 - 15 */
#define INTMASK_HI              (INTCTL_HI + 1) /* PIC mask register IRQ 8 - 15 */
#define EOI             0x20            /* End of interrupt signal */

#define N_WINDOWS       4
#define OUTPUT_WINDOW   0
#define INPUT_WINDOW    1
#define MC_TIME_WINDOW  2
#define AUX_WINDOW      3

#ifndef NULL
#define NULL            0
#endif

#define EVEN            3
#define ODD             1
#define NONE            0

#define DCD         BIT3
#define CTS         BIT5
#define RTS         BIT9
#define DTR         BIT15

#define PgUp        (FUNCT + 73)
#define PgDn        (FUNCT + 81)
#define UpArr       (FUNCT + 72)
#define DnArr       (FUNCT + 80)
#define LArr        (FUNCT + 75)
#define RArr        (FUNCT + 77)

#define ALT_D       (FUNCT + 32)
#define ALT_H       (FUNCT + 35)
#define ALT_F       (FUNCT + 33)
#define ALT_L       (FUNCT + 38)
#define ALT_M       (FUNCT + 50)
#define ALT_N       (FUNCT + 49)
#define ALT_O       (FUNCT + 24)
#define ALT_P       (FUNCT + 25)
#define ALT_Q       (FUNCT + 16)
#define ALT_R       (FUNCT + 19)
#define ALT_S       (FUNCT + 31)
#define ALT_T       (FUNCT + 20)
#define ALT_V       (FUNCT + 47)
#define ALT_X       (FUNCT + 45)

#define F1          (FUNCT + 59)
#define F2          (F1 + 1)
#define F3          (F2 + 1)
#define F4          (F3 + 1)
#define F5          (F4 + 1)
#define F6          (F5 + 1)
#define F7          (F6 + 1)
#define F8          (F7 + 1)
#define F9          (F8 + 1)
#define F10         (F9 + 1)

#define ULDC        201
#define URDC        187
#define LLDC        200
#define LRDC        188
#define HDL         205
#define VDL         186
#define LDT         204
#define RDT         185

#define BLOB        ("\376")            /* display blob for modem ctls */

typedef struct host_que_buf {
        struct  host_que_buf *bh_next;    /* forward thread */
        UINT                  bh_txct;    /* buffer text char count */
        time_t                bh_time;    /* timestamp */
        UTINY                 bh_type;    /* buffer type */
        UTINY                 bh_line;    /* line number */
        UTINY                 bh_flgs;    /* indicator flags */
        UINT                  bh_errs;    /* error flags */
        UTINY                 bh_text[BUF_SIZE];
} BH_;

typedef struct host_qcb {
        struct  host_que_buf *qh_frst;      /* next on queue */
        struct  host_que_buf *qh_last;      /* last on queue */
} QH_;

typedef struct rx_data {
        UTINY   rd_line;            /* Hostess i port number */
        UTINY   rd_flgs;            /* status flags */
        UTINY   rd_data[BUF_SIZE];  /* data area */
} RD_;

typedef struct  {
                int             col;                    /* current column */
                int             row;                    /* current row */
                long    text_color;     /* text color */
                long    bkgd_color;     /* background color */
                short   xl;             /* left column */
                short   xr;             /* right column */
                short   yu;             /* upper row */
                short   yl;             /* lower row */
} WS_;

/** Global Descriptor Table for protected mode data moves                **/
/* Global Descriptor Table is composed of six eight byte descriptors with */
/* the following format:                                                  */
/*    2 bytes - segment length in bytes                                   */
/*    2 bytes - bits 15 - 0 of absolute address                           */
/*    1 byte  - bits 23 - 16 of absolute address                          */
/*    1 byte  - access rights byte                                        */
/*    2 bytes - reserved                                                  */
/* The GDT is organized as a two dimensional array instead of an array    */
/* of structures to guarantee that there are no "holes" as can occur with */
/* structures.                                                            */

typedef struct {
        UINT        gd_sglh;            /* segment length in bytes */
        ULONG       gd_addr;            /* absolute address (bytes 0-2) */
        UINT        gd_rsvd;            /* reserved */
} GDT_;                             /* global descriptor table


                                        /* ***** END OF FILE ***** */
