/**********************************************************************
*  File:    SCRN.C                                                    *
*  Purpose: Comtrol Hostess i host screen utilities                   *
*  Date:    November 19, 1990                                         *
*  Author:  Howard D. Straus                                          *
*  Release: 1.00 - Original release                                   *
***********************************************************************
*                         COPYRIGHT NOTICE                            *
*=====================================================================*
*   Copyright 1990, 1991 by Howard D. Straus and Comtrol Corporation. *
*   All rights reserved.  Use or reproduction by any means of any     *
*   portion of this code contrary to the developers license agreement *
*   is expressly prohibited.                                          *
**********************************************************************/

#include "compiler.h"

#include <conio.h>
#include <ctype.h>
#include <dos.h>
#ifdef  TURBO_C
#include <graphics.h>
#else
#include <graph.h>
struct  rccoord temp;
#endif
#include <string.h>
#include <time.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <fcntl.h>
#include <io.h>
#include <stdio.h>
#include <stdlib.h>

#include "..\hostess\hostess.h"

#include "host.h"
#include "hostfns.h"
#include "hostext.h"

#define PAGE_LENGTH 0x60

char                    *par_str[] = { "     No", "    Odd", "Invalid", "   Even" };
char                    *stp_str[] = { "Invalid", "  1", "1.5", "  2" };
char                    *lns_str[] = { "Closed   ", "Idle     ", "Active   ",
                                                           "In Break ", "Line Fail" };
char            hxd[16][2] = { "0", "1", "2", "3", "4", "5", "6", "7",
                                                           "8", "9", "A", "B", "C", "D", "E", "F" };


/********************************************************************
*  Name:            change_to_window                                *
*  Purpose:         changes the window environment to a new window  *
*  Call:            change_to_window(n);                            *
*  Parameters:                                                      *
*       int n;  the number of the window to which to change         *
*  Returns:         none.                                           *
*  Method:          saves the current window cursor position,       *
*       changes the window boundaries, color specifications, then   *
*       positions the cursor to the last saved cursor position for  *
*       the window.                                                 *
*  Notes:                                                           *
*********************************************************************/

void
change_to_window(n)
        int             n;      /* the window to which to change */
{
        wp->col = GETTEXTCOL(); /* find cursor position */
        wp->row = GETTEXTROW();
        last_window = current_window;   /* save window from which changed */
        if ((n == INPUT_WINDOW) || (n == OUTPUT_WINDOW))
        {       if (saved_ptr != NULL)
                        restore_screen();
        }
        current_window = n;
        wp = &window_p[n];

        WINDOW(wp->yu, wp->xl, wp->yl, wp->xr);
        if (iscolor)
        {   TEXTCOLOR(wp->text_color);
                TEXTBACKGROUND(wp->bkgd_color);
        }
        TEXTPOSITION(wp->row, wp->col);
}


/************************************************************************
*  Name:            hline                                               *
*  Purpose:         draws a horizontal line of characters on the screen *
*  Call:            hline(cs, cm, ce, row, fcol, lcol);                 *
*  Parameters:                                                          *
*       int cs;  the character to display at the beginning of the line  *
*       int cm;  the character comprising the middle of the line        *
*       int ce;  the character to display at the end of the line        *
*       int row;  the row on the screen at which to place the line      *
*       int fcol;  the starting column of the line                      *
*       int lcol;  the ending column of the line                        *
*  Returns:         none.                                               *
*  Method:          positions the cursor to the correct row and column, *
*       then builds the desired string, and outputs it in the desired   *
*       starting position.                                              *
*  Notes:                                                               *
*************************************************************************/

void
hline(cs, cm, ce, row, fcol, lcol)
        int     cs, cm, ce, row, fcol, lcol;
{   int     i, j;
        char    work[81];

        TEXTPOSITION(row, fcol);            /* place the cursor at line start */
        memset(work, cm, lcol - fcol);
        work[0] = cs;                       /* left corner character */
        work[lcol - fcol] = ce;             /* right corner character */
        work[lcol - fcol + 1] = 0;          /* terminate string */
        OUTTEXT(work);
}


/********************************************************************
*  Name:        init_windows
*  Purpose:     set up the screen and window descriptions
*  Call:        init_windows();
*  Parameters:  none.
*  Returns:     none.
*  Method:      draws framing and title information, legend informa-
*       tion.  sets the cursor to the output window.
*  Notes:
********************************************************************/

void
init_windows()
{   int     i;                          /* work variable */

        if (iscolor)                        /* if color screen */
        {   TEXTCOLOR(LIGHTGRAY);           /* define colors */
                TEXTBACKGROUND(BLUE);
                videomemory_addr = MK_FP(0xB800, 0); /* locate color vidwo memory */
        }
        else
        {   videomemory_addr = MK_FP(0xB000, 0); /* locate mono video memory */
        }
        WRAPOFF();                          /* turn off line wrap */
        CLEARSCREEN;                        /* clear the screen */
        vline(ULDC, VDL, LLDC, 1, 1, 25);   /* draw the frame characters */
        vline(URDC, VDL, LRDC, 80, 1, 25);
        hline(ULDC, HDL, URDC, 1, 1, 80);
        hline(LDT, HDL, RDT, 12, 1, 80);
        hline(LDT, HDL, RDT, 23, 1, 80);
        hline(LLDC, HDL, LRDC, 25, 1, 80);
        if (iscolor)
        {   TEXTCOLOR(BLACK);
                TEXTBACKGROUND(CYAN);
        }
        TEXTPOSITION(1, 35);                /* add the titles to the frame */
        OUTTEXT("\265 OUTPUT \306");
        TEXTPOSITION(12, 35);
        OUTTEXT("\265 INPUT \306");
        TEXTPOSITION(23, 23);
        OUTTEXT("\265 MODEM CONTROL AND HOSTESS TIME \306");
        TEXTPOSITION(25, 21);
        OUTTEXT("\265 COMTROL HOSTESS i DEMO PROGRAM V1.0 \306");

        wp = &window_p[0];                  /* describe output screen */
        wp->text_color = YELLOW;
        wp->bkgd_color = BLUE;
        wp->xl = 3;
        wp->xr = 78;
        wp->yu = 2;
        wp->yl = 11;
        wp->row = 1;
        wp->col = 1;

        wp = &window_p[1];                  /* describe input screen */
        wp->text_color = YELLOW;
        wp->bkgd_color = BLUE;
        wp->xl = 3;
        wp->xr = 78;
        wp->yu = 13;
        wp->yl = 22;
        wp->row = 1;
        wp->col = 1;

        wp = &window_p[2];                  /* describe modem control screen */
        wp->text_color = LIGHTGRAY;
        wp->bkgd_color = BLUE;
        wp->xl = 3;
        wp->xr = 78;
        wp->yu = 24;
        wp->yl = 24;
        wp->row = 1;
        wp->col = 1;

        wp = &window_p[3];                  /* describe help (AUX) window */
        wp->text_color = BLACK;
        wp->bkgd_color = CYAN;
        wp->xl = 11;
        wp->xr = 69;
        wp->yu = 6;
        wp->yl = 19;
        wp->row = 1;
        wp->col = 1;

        current_window = 0;                 /* initialize current window */

        wp = &window_p[0];
        for (i = 0; i < MC_TIME_WINDOW; i++)
        {   change_to_window(i);
                CLEARWINDOW;
                wp->col = GETTEXTCOL();
                wp->row = GETTEXTROW();
        }
        change_to_window(MC_TIME_WINDOW);   /* initialize text in control area */
        TEXTPOSITION(1, 2);
        OUTTEXT(
                "Line  1  Loop OFF  RTS x   CTS x   DCD x   DTR x           Time:");
        change_to_window(0);                /* default to window 0 */
        WRAPON();                           /* turn on text wrapping */
}


/********************************************************************
*  Name:        dump_memory                                         *
*  Purpose:     show the contents of desired addresses in DP-RAM    *
*       until an ESC character is received.
*  Call:        dump_memory();                                      *
*  Parameters:  none.                                               *
*  Returns:     none.                                               *
*  Method:      dumps the contents of a page of dual-ported RAM to  *
*       the AUX page.                                               *
*  Notes:                                                           *
********************************************************************/

void
dump_memory()
{   static  long    last_address = 0L;
        long            new_address;
        int             row, col;

        save_screen();
        if (last_address == 0L)
                last_address += base_address;
        change_to_window(AUX_WINDOW);
        CLEARWINDOW;
        OUTTEXT("Dual-Ported RAM Address: ");
        row = GETTEXTROW();
        col = GETTEXTCOL();
        show_address(last_address);
        dump_memory_page(last_address);
        TEXTPOSITION(row, col);
        while ((new_address = get_dp_ram_address(last_address)) >= 0)
        {   dump_memory_page(last_address = base_address + new_address);
                TEXTPOSITION(row, col);
        }
        restore_screen();
}


/********************************************************************
*  Name:        dump_memory_page                                    *
*  Purpose:     show the contents of a page of DP-RAM               *
*  Call:        dump_memory_page(addr);                             *
*  Parameters:                                                      *
*       long addr;  flat (absolute) address of DP-RAM page to be    *
*               dumped to the screen.  Page is PAGE_LENGTH bytes    *
*               long.                                               *
*  Returns:     none.                                               *
*  Method:      dumps the contents of a page of dual-ported RAM to  *
*       the AUX window.                                             *
*  Notes:                                                           *
********************************************************************/

void
dump_memory_page(addr)
        long    addr;
{   UTINY   mem_page[PAGE_LENGTH + 10];
        UTINY   *cpc, *cph;
        char    s[2];
        long    i, j;

        s[1] = 0;
        CLEARWINDOW;
        OUTTEXT("Dual-Ported RAM Address: ");
        show_address(addr);
        read_from_hcb((char far *)mem_page, addr, (PAGE_LENGTH + 5) >> 1);
        cph = cpc = mem_page;
        for (i = addr; i < addr + PAGE_LENGTH; i += 8)
        {   OUTTEXT("\r\n");
                show_address(i);
                OUTTEXT(" ");
                for (j = 0; j < 8; j++)
                {   OUTTEXT(" ");
                        OUTTEXT(hxd[*cph >> 4]);
                        OUTTEXT(hxd[*cph++ & 0xF]);
                }
                OUTTEXT("  ");
                for (j = 0; j < 8; j++)
                {   s[0] = *cpc++;
                        if (!isprint(s[0]))
                                s[0] = '.';
                        OUTTEXT(s);
                }
        }
}


/********************************************************************
*  Name:        get_dp_ram_address                                  *
*  Purpose:     return the address selected by the user.            *
*  Call:        addr = get_dp_ram_address(last);                    *
*  Parameters:                                                      *
*       long last;  the last address displayed on the screen.       *
*  Returns:                                                         *
*       long addr;  the new address, based on the last address plus *
*               user keystrokes.                                    *
*  Method:      returns a flat (absolute) offset into the DP-RAM    *
*       based on the last address plus user keystrokes.  The ad-    *
*       dress is displayed as it is being entered.  The result is   *
*       returned only upon a CR (Enter) or Esc key.                 *
*                                                                   *
*       hex digits:  digit value shifted in on right of result      *
*       backspace:   rightmost hex digit shifted off                *
*       up arrow:    last - 8 (line up)                             *
*       down arrow:  last + 8 (line down)                           *
*       left arrow:  last - 2 (word backward)                       *
*       right arrow: last + 2 (word forward)                        *
*       page up:     last - PAGE_LENGTH                             *
*       page down:   last + PAGE_LENGTH                             *
*       ESC:         -1L (exit code)                                *
*       CR:          last                                           *
********************************************************************/

long
get_dp_ram_address(last)
        long    last;
{   UINT    c;
        long    intermediate;
        int     col, row;

        row = GETTEXTROW();
        col = GETTEXTCOL();
        intermediate = last & 0x1FFFFL;
        c = 0;
        while ((c != CR) && (c != ESC))
        {   TEXTPOSITION(row, col);
                show_address(intermediate + base_address);
                c = get_key();
                switch (c) {

                case ' ':           /* clear intermediate */
                        intermediate = 0;
                        break;
                case '\b':          /* rubout last digit */
                        intermediate >>= 4;
                        break;
                case ESC:           /* exit routine */
                        return(-1L);
                case CR:            /* return number */
                        return(intermediate);
                case PgUp:          /* prior page */
                        return(max(intermediate - PAGE_LENGTH, 0L));
                case PgDn:          /* next page */
                        return(intermediate + PAGE_LENGTH);
                case LArr:
                        return(max(intermediate - 2, 0L));
                case RArr:
                        return(intermediate + 2);
                case UpArr:         /* scroll up */
                        return(max(intermediate - 8, 0L));
                case DnArr:         /* scroll down */
                        return(intermediate + 8);
                default:
                        if (isxdigit(c = toupper(c)))
                        {   intermediate = (intermediate & 0xFFFF) << 4;
                                intermediate += (c - '0' <= 9 ? c - '0' : c - 'A' + 10);
                        }
                        break;
                }
        }
        return(-1);
}


/********************************************************************
*  Name:        restore_screen                                      *
*  Purpose:     restore the contents of a screen after displaying   *
*       a help screen                                               *
*  Call:        restore_screen();                                   *
*  Parameters:  none.                                               *
*  Returns:     none.                                               *
*  Method:      copies the contents of the parameter buffer to the  *
*       video buffer.  In Turbo C, there is a function for this, in *
*       MSC it must be done explicitly with a loop.                 *
*  Notes:                                                           *
********************************************************************/

void
restore_screen()
{   int             *sp;
        int     far     *lp;
        UINT            size;
        UINT            i;

#ifdef  TURBO_C
        puttext(1, 1, 80, 25, saved_ptr);
#else
        sp = (int *)saved_ptr;
        lp = (int far *)videomemory_addr;
        for (i = 0; i < 2000; i++)
                *lp++ = *sp++;
#endif
        saved_ptr = NULL;
        in_aux = 0;
}


/********************************************************************
*  Name:        save_screen                                         *
*  Purpose:     save the current contents of the CRT screen         *
*  Call:        save_screen();                                          *
*  Parameters:  nons.                                               *
*  Returns:     none.                                               *
*  Method:      saves the screen to a fixed buffer.                                 *
*  Notes:                                                           *
********************************************************************/

void
save_screen()
{       UINT            i;
        int                             *ip;
        int             far             *lp;
        int                             *sp;

        sp = (int *)screen_buffer;
#ifdef  TURBO_C
        gettext(1, 1, 80, 25, sp);
#else
        lp = (int far *)videomemory_addr;
        for (i = 0; i < 2000; i++)
                *sp++ = *lp++;
#endif
        saved_ptr = screen_buffer;
}


/********************************************************************
*  Name:        show_address                                        *
*  Purpose:     show a flat (absolute) address at the current       *
*       cursor position.                                            *
*  Call:        show_address(addr);                                 *
*  Parameters:                                                      *
*       long addr;  the long integer address to be displayed at the *
*           current cursor position in the window.                  *
*  Returns:     none.                                               *
*  Method:      converts the address to a string, then places the   *
*       string in the current window at the current cursor locarion *
********************************************************************/

void
show_address(addr)
        long    addr;
{   char    str[20];

        OUTTEXT(strupr(ltoa(addr, str, 16)));
}


/********************************************************************
*  Name:        show_help_screen                                    *
*  Purpose:     show a pop-up help screen with the contents of a    *
*       help file displayed in it.                                  *
*  Call:        show_help_screen(level);                            *
*  Parameters:                                                      *
*       char *level;  character string denoting the file neae to    *
*           display.  e. g.: if *level is "H0", the file "H0.HLP"   *
*           will be displayed.                                      *
*  Returns:     none.                                               *
*  Method:      draws a freme in the middle of the screen, then     *
*       reads the desired file, line by line, and displays it in    *
*       the frame.  Up to 14 lines of up to 59 characters will be   *
*       displayed.                                                  *
*  Notes:       the caller must save the screen before calling this *
*       function, and restore it when finished.                     *
********************************************************************/

void
show_help_screen(level)
        UTINY   *level;
{   char    far     *cp, *dp;
        char            *sp, *tp;
        FILE            *help_file;
        char            help_file_name[64];
        char            line[80];
        int                             save;

        save_screen();                                          /* save the former screen */
        change_to_window(AUX_WINDOW);           /* save cursor position */
        strcpy(help_file_name, level);          /* construct file name */
        strcat(help_file_name, ".HLP");
        WINDOW(1, 1, 25, 80);               /* draw frame */
        if (iscolor)
        {   TEXTCOLOR(WHITE);
                TEXTBACKGROUND(CYAN);
        }
        vline(219, 219, 219, wp->xl - 1,  wp->yu - 1, wp->yl + 1);
        vline(219, 219, 219, wp->xr + 1,  wp->yu - 1, wp->yl + 1);
        hline(219, 223, 219, wp->yu - 1,  wp->xl - 1, wp->xr + 1);
        hline(219, 220, 219, wp->yl + 1,  wp->xl - 1, wp->xr + 1);

        change_to_window(AUX_WINDOW);           /* establish window */
        CLEARWINDOW;                                            /* clear the new window */
        if ((help_file = fopen(help_file_name,"r")) == NULL) /* open file */
        {   OUTTEXT("No help file available for this level.");
                return;
        }
        while(!feof(help_file))                         /* read entire file */
        {   fgets(line, 60, help_file);
                if (!strncmp(line, "*EOF*", 5))
                        break;
                OUTTEXT(line);                                  /* put line to panel */
#ifdef  TURBO_C
                OUTTEXT("\r");
#endif
        }
        fclose(help_file);                                      /* close text file */
        TEXTPOSITION(1, 1);                                     /* put cursor at ULC */
        in_aux = HELP_FILE;                                     /* flag contents of window */
}


/********************************************************************
*  Name:        format_device_table                                                                     *
*  Purpose:     interpret the device table to the aux window            *
*  Call:        format_device_table();                                  *
*  Parameters:  none.                                               *
*  Returns:     none.                                               *
*  Method:      interprets the current device table to the aux-         *
*               iliary window.                                                                                          *
*  Notes:                                                                                                               *
********************************************************************/

void
format_device_table()
{   char            line[80];
        char                    *cp;
        DV_                             *dv;
        LC_                             *lc;

        dv = &dct;                                                      /* device control table */
        lc = &lct;                                                      /* line control table */

        TEXTPOSITION(1, 1);                                     /* put cursor at ULC */
        sprintf(line, "                Device Table %d (%Np) Contents\n\n\r",
                        lc->lc_lnno + 1, lc->lc_dvad);

        OUTTEXT(line);                                          /* output title line */
        sprintf(line, "Next Dev Tbl: %Np   Tx Brk Start: %8lX  Tx Break : %2X\n\r",
                                dv->dv_next, dv->dv_tbks, dv->dv_brks);
        OUTTEXT(line);                                          /* write first line */
        sprintf(line,
                          "  Line Table: %Np         Flags : %02X        %4d Bps.\n\r",
                                dv->dv_lcad, dv->dv_flgs,
                                (dv->dv_spd == 0xFFFF ? 9600 : dv->dv_spd / 10));
        OUTTEXT(line);
        sprintf(line,
                          "Command Port: %4X      Last RR0 : %2X     %3d Bits/char\n\r",
                                dv->dv_cmdp, dv->dv_lrr0, dv->dv_bits);
        OUTTEXT(line);
        sprintf(line,
                          "   Data Port: %4X      Tx %s      %s Parity\n\r",
                                dv->dv_datp, (dv->dv_txe ? "Enabled " : "Disabled"),
                                par_str[dv->dv_par]);
        OUTTEXT(line);
        sprintf(line,
                          "Rx Brk Start: %8lX       RTS : %s    %s Stop Bits\n\r",
                                dv->dv_rbks, (dv->dv_rts ? "ON " : "OFF"),
                                stp_str[dv->dv_stpb >> 2]);
        OUTTEXT(line);
        sprintf(line,
                          "                             DTR : %s      %sLooped\n\r",
                                (dv->dv_dtr ? "ON " : "OFF"),
                                (dv->dv_loop ? "" : "Not "));
        OUTTEXT(line);
}


/********************************************************************
*  Name:        show_device_table                                                                       *
*  Purpose:     show a pop-up screen with the interpreted contents  *
*       of the current device control table.                        *
*  Call:        show_device_table();                                    *
*  Parameters:  none.                                               *
*  Returns:     none.                                               *
*  Method:      draws a freme in the middle of the screen, then     *
*       interprets the device table contents from the last status       *
*       message.                                                                                                        *
*  Notes:                                                                                                               *
********************************************************************/

void
show_device_table()
{   char            line[80];
        char                    *cp;
        DV_                             *dv;
        LC_                             *lc;

        if (in_aux == 0)
        {       save_screen();                                          /* save the display */
                change_to_window(AUX_WINDOW);           /* save cursor position */
                WINDOW(1, 1, 25, 80);               /* draw frame */
                if (iscolor)
                {   TEXTCOLOR(WHITE);
                        TEXTBACKGROUND(CYAN);
                }
                vline(219, 219, 219, wp->xl - 1,  wp->yu - 1, wp->yl + 1);
                vline(219, 219, 219, wp->xr + 1,  wp->yu - 1, wp->yl + 1);
                hline(219, 223, 219, wp->yu - 1,  wp->xl - 1, wp->xr + 1);
                hline(219, 220, 219, wp->yl + 1,  wp->xl - 1, wp->xr + 1);
        }
        change_to_window(AUX_WINDOW);

        if (in_aux != DEVICE_TABLE)
                CLEARWINDOW;                            /* clear the new window */
        in_aux = DEVICE_TABLE;                          /* flag contents */
        format_device_table();                          /* show the table */
        TEXTPOSITION(1, 1);                                     /* put cursor at ULC */
}


/********************************************************************
*  Name:        format_line_table                                                                       *
*  Purpose:     interpret the line table into the current window        *
*  Call:        format_line_table();                                    *
*  Parameters:  none.                                               *
*  Returns:     none.                                               *
*  Method:      interprets the line table into the current window       *
*  Notes:               the caller must have first saved the screen, then       *
*               cleared an auxiliary window before calling this routine         *
********************************************************************/

void
format_line_table()
{   char            line[80];
        char                    *cp;
        LC_                             *lc;
        DV_                             *dv;
        int                             x, y;

        lc = &lct;                                                      /* develop a pointer to table */
        dv = &dct;                                                      /* get device control table address */

        sprintf(line, "               Line Table %d (%Np) Contents\n\n\r",
                                lc->lc_lnno + 1, dv->dv_lcad);
        OUTTEXT(line);                                          /* output title line */
        sprintf(line, "Next LCT %Np    Out Bufs Ct %5d\n\r",
                                lc->lc_next, lc->lc_obct);
        OUTTEXT(line);                                          /* write first line */
        sprintf(line, "DCT Addr %Np    Tx Que Frst %Fp   Rx Flgs %04X\n\r",
                                lc->lc_dvad, lc->lc_txqc.qc_frst, lc->lc_rxfl);
        OUTTEXT(line);
        sprintf(line, "Err Flgs %5d   Tx Que Last %Fp   Tx %s\n\r",
                                lc->lc_errs, lc->lc_txqc.qc_last, lns_str[lc->lc_txst]);
        OUTTEXT(line);
        sprintf(line, "   Tx Ct %5d   Curr Tx Buf %Fp   Rx %s\n\r",
                                lc->lc_txcc, lc->lc_ctxb, lns_str[lc->lc_rxst]);
        OUTTEXT(line);
        sprintf(line, "   Rx Ct %5d   Curr Rx Buf %Fp   Tx Event %2d\n\r",
                                lc->lc_rxcc, lc->lc_crxb, lc->lc_txev);
        OUTTEXT(line);
        sprintf(line, "Rxd Msgs %5d   Next Tx Chr %Fp   Rx Event %2d\n\r",
                                lc->lc_trxm, lc->lc_nxtc, lc->lc_rxev);
        OUTTEXT(line);
        sprintf(line, "1st Rxd @%6ld  Next Rx Chr %Fp\n\r",
                                lc->lc_tfrx, lc->lc_nxrc);
        OUTTEXT(line);
        x = GETTEXTCOL();
        y = GETTEXTROW();
        memset(line, 0, sizeof(line));
        memset(line, ' ', wp->xr - wp->xl - 2);
        strcat(line, "\r\n");
        OUTTEXT(line);
        OUTTEXT(line);
        TEXTPOSITION(y, x);
        if (lc->lc_crxb != NULL)
        {       memset(line, 0, sizeof(line));
                strcpy(line, "crxb: ");
                read_from_hcb((char far *)&line[6], base_address +
                        norm(lc->lc_crxb) + BF_HDR, 26);
                if ((cp = strchr(line, '\n')) != NULL)
                        *cp = 0;
                strcat(line, "\r\n");
                OUTTEXT(line);
        }
        if (lc->lc_ctxb != NULL)
        {       memset(line, 0, sizeof(line));
                strcpy(line, "ctxb: ");
                read_from_hcb((char far *)&line[6], base_address +
                        norm(lc->lc_ctxb) + BF_HDR, 26);
                if ((cp = strchr(line, '\n')) != NULL)
                        *cp = 0;
                strcat(line, "\r\n");
                OUTTEXT(line);
        }
}


/********************************************************************
*  Name:        show_line_table                                                                         *
*  Purpose:     show a pop-up screen with the interpreted contents  *
*       of the current line control table.                          *
*  Call:        show_line_table();                                      *
*  Parameters:  none.                                               *
*  Returns:     none.                                               *
*  Method:      draws a freme in the middle of the screen, then     *
*       interprets the line table contents from the last status         *
*       message.                                                                                                        *
*  Notes:       the caller must save the screen before calling this *
*       function, and must restore it when the help is no longer    *
*       sired.  This function destroys the current display.         *
********************************************************************/

void
show_line_table()
{   char            line[80];
        int                             save;
        int                             *sp;
        LC_                             *lc;
        DV_                             *dv;
        char                    *cp;

        if (in_aux == 0)                                        /* if aux window not up */
        {       save_screen();                                          /* save the display */
                change_to_window(AUX_WINDOW);           /* save cursor position */
                WINDOW(1, 1, 25, 80);               /* draw frame */
                if (iscolor)
                {   TEXTCOLOR(WHITE);
                        TEXTBACKGROUND(CYAN);
                }
                vline(219, 219, 219, wp->xl - 1,  wp->yu - 1, wp->yl + 1);
                vline(219, 219, 219, wp->xr + 1,  wp->yu - 1, wp->yl + 1);
                hline(219, 223, 219, wp->yu - 1,  wp->xl - 1, wp->xr + 1);
                hline(219, 220, 219, wp->yl + 1,  wp->xl - 1, wp->xr + 1);

        }

        change_to_window(AUX_WINDOW);           /* establish window */
        if (in_aux != LINE_TABLE)
                CLEARWINDOW;
        in_aux = LINE_TABLE;                            /* flag line table in window */
        format_line_table();                            /* interpret the line table */
        TEXTPOSITION(1, 1);                                     /* leave cursor at ULC */
}


/********************************************************************
*  Name:        show_input
*  Purpose:     show the string in the next position of the input
*               window on the screen
*  Call:        show_input(buffer);
*  Parameters:
*       char *buffer;  the buffer of characters to display in the
*               input window.
*  Returns:     none.
*  Method:      changes windows to the input window, locates the
*       cursor at the correct position, and outputs the string to
*       the screen.
*  Notes:
********************************************************************/

void
show_input(buffer)
        char            *buffer;
{
        if (current_window != INPUT_WINDOW)
                change_to_window(INPUT_WINDOW);
        OUTTEXT(buffer);
}


/********************************************************************
*  Name:            show_modem_controls
*  Purpose:         display RTS, CTS, DCD and DTR signal conditions
*           on the last line of the formatted screen.
*  Call:            show_modem_controls(mcw);
*  Parameters:
*       int mcw;  modem control signal status bits (1 = ON)
*                   BIT     SIGNAL
*                    3       DCD
*                    5       CTS
*                    9       RTS
*                   15       DTR
*  Returns:         none.
*  Method:          shows as single 'blobs' each signal.  If the cor-
*       responding bit is set, the blob will show, if not, it will be
*       erased.
*  Notes:           the positions are hard-coded into the function.
*       if desirable, they could be made #defines for convenience.
********************************************************************/

void
show_modem_controls(mcw)
        int             mcw;        /* bits to display */
{   char            cvstr[20];  /* integer to ASCII string conversion */

        if (current_window != MC_TIME_WINDOW)
                change_to_window(MC_TIME_WINDOW);
        if (iscolor)
                TEXTCOLOR(YELLOW);
        TEXTPOSITION(1, (hcb_port > 8 ? 7 : 8));
        OUTTEXT(itoa(hcb_port + 1, cvstr, 10));
        TEXTPOSITION(1, 16);
        OUTTEXT(loopback_mode ? "ON " : "OFF");
        TEXTPOSITION(1, 25);
        if (mcw & RTS)
                OUTTEXT(BLOB);
        else
                OUTTEXT(" ");
        TEXTPOSITION(1, 33);
        if (mcw & CTS)
                OUTTEXT(BLOB);
        else
                OUTTEXT(" ");
        TEXTPOSITION(1, 41);
        if (mcw & DCD)
                OUTTEXT(BLOB);
        else
                OUTTEXT(" ");
        TEXTPOSITION(1, 49);
        if (mcw & DTR)
                OUTTEXT(BLOB);
        else
                OUTTEXT(" ");
}


/********************************************************************
*  Name:        show_output
*  Purpose:     show the string in the next position of the output
*               window on the screen
*  Call:        show_output(buffer);
*  Parameters:
*       char *buffer;  the buffer of characters to display in the
*               output window.
*  Returns:     none.
*  Method:      changes windows to the output window, locates the
*       cursor at the correct position, and outputs the string to
*       the screen.
*  Notes:
********************************************************************/

void
show_output(buffer)
        char            *buffer;
{
        if (current_window != OUTPUT_WINDOW)
                change_to_window(OUTPUT_WINDOW);
        OUTTEXT(buffer);
}


/********************************************************************
*  Name:            show_time
*  Purpose:         displays the time in the right hand end of the
*           modem control/time window.
*  Call:            show_time(tm);
*  Parameters:
*       time_t tm;  the DOS formatted time in seconds since the
*           reference time (00:00, 1/1/1980).
*  Returns:         none.
*  Method:          converts the time to a string, then extracts the
*       relevant characters, and places them in the correct window.
*  Notes:
********************************************************************/

void
show_time(tm)
        time_t          tm;
{   int             save;
        char            time_str[20];

        change_to_window(MC_TIME_WINDOW);
        TEXTPOSITION(1, 68);                /* position at time slot */
        if (iscolor)                        /* if color screen */
                TEXTCOLOR(YELLOW);              /* set the text color */
        if (tm == 0L)                       /* if initialized time */
                OUTTEXT("--:--:--");            /* put out indicator string */
        else
        {   strcpy(time_str, &(ctime(&tm))[11]); /* convert to HH:MM:SS */
                time_str[8] = 0;                /* nul-terminate string */
                OUTTEXT(time_str);              /* output to the screen */
        }
}


/********************************************************************
*  Name:            vline
*  Purpose:         draw a vertical line of characters on the screen
*  Call:            vline(cs, cm, ce, col, frow, lrow);
*  Parameters:
*       int cs;  the character to display at the beginning of the line
*       int cm;  the character comprising the middle of the line
*       int ce;  the character to display at the end of the line
*       int col;  the column at which to draw the line
*       int frow;  the first row of the vertical line
*       int lrow;  the last row of the vertical line
*  Returns:         none.
*  Method:          positions the cursor at the first row and column
*       specified, places characters in successive rows until the last
*       row is reached.
*  Notes:
********************************************************************/

void
vline(cs, cm, ce, col, frow, lrow)
        int     cs, cm, ce, col, frow, lrow;
{   int     i, j;
        char    work[4];

        work[0] = cs;                   /* top of line character */
        work[1] = 0;                    /* make it a string */
        for (i = frow; i < lrow; i++)   /* output as many as requested */
        {   TEXTPOSITION(i, col);       /* position the cursor */
                OUTTEXT(work);              /* output the character */
                work[0] = cm;               /* set character for the next output */
        }
        TEXTPOSITION(lrow, col);        /* set for the bottom character */
        work[0] = ce;                   /* set character string */
        OUTTEXT(work);                  /* output the bottom character */
}


                                        /* ***** END OF FILE ***** */
